@@ -73,7 +73,13 @@ class Project:  # pylint: disable=too-many-instance-attributes
             )
         else:
             loader = package_loader
-        self.env: Environment = Environment(loader=loader, trim_blocks=True, lstrip_blocks=True)
+        self.env: Environment = Environment(
+            loader=loader,
+            trim_blocks=True,
+            lstrip_blocks=True,
+            extensions=["jinja2.ext.loopcontrols"],
+            keep_trailing_newline=True,
+        )
 
         self.project_name: str = config.project_name_override or f"{utils.kebab_case(openapi.title).lower()}-client"
         self.project_dir: Path = Path.cwd()
@@ -123,7 +129,7 @@ class Project:  # pylint: disable=too-many-instance-attributes
         """Update an existing project"""
 
         if not self.package_dir.is_dir():
-            raise FileNotFoundError()
+            return [GeneratorError(detail=f"Directory {self.package_dir} not found")]
         print(f"Updating {self.package_name}")
         shutil.rmtree(self.package_dir)
         self._create_package()
@@ -267,7 +273,9 @@ class Project:  # pylint: disable=too-many-instance-attributes
             encoding=self.file_encoding,
         )
 
-        endpoint_template = self.env.get_template("endpoint_module.py.jinja")
+        endpoint_template = self.env.get_template(
+            "endpoint_module.py.jinja", globals={"isbool": lambda obj: obj.get_base_type_string() == "bool"}
+        )
         for tag, collection in endpoint_collections_by_tag.items():
             tag_dir = api_dir / tag
             tag_dir.mkdir()
@@ -281,7 +289,12 @@ class Project:  # pylint: disable=too-many-instance-attributes
 
             for endpoint in collection.endpoints:
                 module_path = tag_dir / f"{utils.PythonIdentifier(endpoint.name, self.config.field_prefix)}.py"
-                module_path.write_text(endpoint_template.render(endpoint=endpoint), encoding=self.file_encoding)
+                module_path.write_text(
+                    endpoint_template.render(
+                        endpoint=endpoint,
+                    ),
+                    encoding=self.file_encoding,
+                )
 
 
 def _get_project_for_url_or_path(  # pylint: disable=too-many-arguments
@@ -368,12 +381,12 @@ def _load_yaml_or_json(data: bytes, content_type: Optional[str]) -> Union[Dict[s
         try:
             return json.loads(data.decode())
         except ValueError as err:
-            return GeneratorError(header="Invalid JSON from provided source: {}".format(str(err)))
+            return GeneratorError(header=f"Invalid JSON from provided source: {err}")
     else:
         try:
             return yaml.safe_load(data)
         except yaml.YAMLError as err:
-            return GeneratorError(header="Invalid YAML from provided source: {}".format(str(err)))
+            return GeneratorError(header=f"Invalid YAML from provided source: {err}")
 
 
 def _get_document(*, url: Optional[str], path: Optional[Path]) -> Union[Dict[str, Any], GeneratorError]:
